@@ -1,4 +1,4 @@
-/* $Id: filesystem.h,v 1.1.1.1 1998/07/29 15:14:27 proff Exp $ */
+/* $Id: filesystem.h,v 1.2 2000/02/19 11:16:16 proff Exp $ */
 
 #ifndef FILESYSTEM_H
 #define FILESYSTEM_H
@@ -16,5 +16,8 @@
 #ifdef HAVE_SYS_STATVFS_H
 #  include <sys/statvfs.h>
 #endif
+#ifdef HAVE_SYS_STATFS_H
+#  include <sys/statfs.h>
+#endif
 #include <dirent.h>
 #endif
