@@ -1,10 +1,14 @@
-/* $Id: standard.h,v 1.2 1998/08/02 20:30:34 proff Exp $
+/* $Id: standard.h,v 1.4 2000/02/19 16:20:53 proff Exp $
  * $Copyright$
  */
 
 #ifndef STANDARD_H
 #define STANDARD_H
 
+#define _X_OPEN_SOURCE
+#define __USE_XOPEN
+#define __USE_XOPEN_EXTENDED
+
 #include "nconf.h"
 #include "ntypes.h"
 
@@ -18,12 +22,22 @@
 #ifdef HAVE_STDARG_H
 #  include <stdarg.h>
 #endif
+#ifdef HAVE_SYS_UNISTD_H
+#  include <sys/unistd.h>
+#endif
 #include <sys/types.h>
 #ifdef HAVE_SYS_PARAM_H
 #  include <sys/param.h>
 #endif
 #include <errno.h>
+#ifndef __USE_GNU
+#  define __USE_GNU
+#  define USE_GNU_SET
+#endif
 #include <string.h>
+#ifdef USE_GNU_SET
+#  undef __USE_GNU
+#endif
 #include <syslog.h>
 #include <ctype.h>
 #include <sys/times.h>
