@@ -5,7 +5,7 @@
 #ifndef DBZ_H
 #define DBZ_H
 
-#include "config.h"
+#include "../config.h"
 
 /* for dbm and dbz */
 typedef struct
