@@ -28,10 +28,8 @@ struct strStack
 /* we don't do the *x==*y trick, as it doesn't take kindly to functions */
 #define strEq(x,y) (strcmp((x), (y)) == 0)
 #define strnEq(x,y,z) (strncmp((x), (y), (z)) == 0)
-#ifdef HAVE_STRCASECMP
-#  define strCaseEq(x,y) (strcasecmp((x), (y)) == 0)
-#  define strnCaseEq(x,y,z) (strncasecmp((x), (y), (z)) == 0)
-#endif
+#define strCaseEq(x,y) (strcasecmp((x), (y)) == 0)
+#define strnCaseEq(x,y,z) (strncasecmp((x), (y), (z)) == 0)
 
 #define strListAdd(x,y) lp_strListAdd ((x), (y), __FILE__ ":" STR(__LINE__))
 #define strStackAdd(x,y) lp_strStackAdd ((x), (y), __FILE__ ":" STR(__LINE__))
