@@ -1,14 +1,15 @@
-/* $Id: article.c,v 1.3 1998/08/04 20:18:52 proff Exp $ */
+/* $Id: article.c,v 1.8 2002/03/29 09:47:55 proff Exp $ */
 
 #include "nglobal.h"
 #include "mmap.h"
 
-#include "xover.h"
-#include "history.h"
 #include "acc.h"
-#include "group.h"
 #include "filter.h"
+#include "group.h"
+#include "history.h"
 #include "lock.h"
+#include "nlist.h"
+#include "xover.h"
 
 #include "article.h"
 
@@ -22,7 +23,7 @@ EXPORT void rfc822lower (char *s)
 		*s = tolower (*s);
 }
 
-EXPORT char *NewsDate (time_t t)
+EXPORT char *newsDate (time_t t)
 {
 	static char buf[80];
 	struct tm *tm = localtime (&t);
@@ -41,7 +42,7 @@ static void art_hilo(char *path)
 	return;
     *p = '\0';
     strExchange(path, '/', '.');
-    n = newsgroup_find_add(path, NULL, FALSE);
+    n = newsgroupFindAddLock(path, NULL, TRUE);
     if (!n)
 	logd (("art_hilo(): no such group '%s'", path));
     strExchange(path, '.', '/');
@@ -53,9 +54,7 @@ static void art_hilo(char *path)
 	n->lo = artno;
     if (n->hi < artno)
 	n->hi = artno;
-#ifdef READ_LOCKS
-    n->read_locks--;
-#endif
+    newsgroupUnlockWrite(n);
 }
 
 
@@ -196,7 +195,7 @@ static bool crosspost (struct strStack *stack, char *path, enum cmds type, bool
 		return FALSE;
 	}
 	artfile = links->head->data;
-	fd = open (artfile, O_WRONLY | O_EXCL | O_CREAT, 0664);
+	fd = open (artfile, O_WRONLY | O_EXCL | O_CREAT, 0000 /* tag for incompleteness */ );
 	if (fd == -1)
 	{
 		if (errno == EEXIST)
@@ -206,7 +205,7 @@ static bool crosspost (struct strStack *stack, char *path, enum cmds type, bool
 			Stats->article_unlinked++;
 		} else
 			blddir (artfile);
-		fd = open (artfile, O_WRONLY | O_EXCL | O_CREAT, 0664);
+		fd = open (artfile, O_WRONLY | O_EXCL | O_CREAT, 0000 );
 		if (fd<0)
 		{
 			loge (("couldn't create article %s", artfile));
@@ -222,6 +221,7 @@ static bool crosspost (struct strStack *stack, char *path, enum cmds type, bool
 		strListFree (links);
 		return FALSE;
 	}
+	fchmod(fd, 0664 /* complete */);
 	close (fd);
 	art_hilo(artfile);
 
@@ -238,7 +238,7 @@ static bool crosspost (struct strStack *stack, char *path, enum cmds type, bool
 			ok = hisAdd (msgid, buf);
 			logd (("adding <%s>:%s to %s (%s)", msgid, buf, con->historyFile, ok ? "suceeded" : "failed"));
 		} else
-			logw (("suspect path: '%.256'", buf));
+			logw (("suspect path: '%.256s'", buf));
 		if (p)
 			*p = '_';
 	}
@@ -317,7 +317,7 @@ EXPORT char *find_header_arg(char *s, char *hdr, int len)
 	char *p;
 	for (;;s=p+1)
 	{
-		p=len? strncasestr(s, hdr, len): strcasestr(s, hdr);
+		p=len? strnCaseStr(s, hdr, len): strCaseStr(s, hdr);
 		if (!p)
 			return NULL;
 		if (p==s_orig)
@@ -383,25 +383,6 @@ static bool auth_article (char *art, int hlen, char *msgid)
 
 /* This routine just shoves out a dummy art with the stats in it */
 
-static bool dummyStatsArticle(struct command *cmd)
-{
-	enum cmds type=cmd->val;
-
-	emit_follows(type, 0, con->statsArticleID, 1);
-	if (type==c_head || type == c_article) {
-		emit("Newsgroups: nntpcache.statistics\r\n");
-		emitf("Message-ID: <%s>\r\n", con->statsArticleID);
-		emitf("Date: %s\r\n", NewsDate(time(NULL)));	
-		emit("Subject: NNTPCACHE Statistics\r\n");
-		emit("From: NNTPCACHE Statistics <nntpcache>\r\n");
-	}
-	if (type==c_body || type == c_article) {
-		PrintStats();
-	}
-	emit(".\r\n");
-	return TRUE;
-}
-
 static void bad_article(struct server_cfg *scfg, enum cmds type)
 {
     switch (type)
@@ -453,8 +434,8 @@ static void good_article(struct server_cfg *scfg, enum cmds type)
  * waiting until we have a complete article in memory before speaking
  * to the client i.e we try to minimise cache latency as much as is
  * possible. This function has "evolved" over time, and needs a
- * complete re-write -- however it works, and its fast,
- * so one isn't game to touch it) */
+ * complete re-write -- however it works, and it's fast,
+ * so one's game to touch it) */
 
 EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 {
@@ -545,8 +526,6 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 					}
 				}
 			}
-			else if (!strcmp(msgid, con->statsArticleID))
-				return(dummyStatsArticle(cmd));
 		} else /* ahh, the simple alternative (!getbyid) */
 		{
 			if (type == c_head || type == c_stat)
@@ -563,7 +542,9 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 				if (type == c_head || type == c_stat)
 				{
 					/* no _head, try for article */
-					*strrchr (artfile, '_') = '\0';
+					char *p;
+					assert ((p = strrchr (artfile, '_')));
+					*p = '\0';
 					real_head_file = FALSE;
 					fd = open (artfile, O_RDONLY);
 				} else  /* c_article or c_body */
@@ -574,12 +555,10 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 					if (fd >= 0)
 					{
 						struct stat st;
-						locksh (fd);
 						if (fstat (fd, &st) == 0)
 						{
 							int len = st.st_size;
 							char *buf = (char *)mmap (0, len, PROT_READ, MAP_PRIVATE, fd, 0); /* slurrrup */
-							lockun (fd);
 							if (buf!=(char *)-1)
 							{
 								int len = st.st_size;
@@ -589,8 +568,7 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 									loge (("couldn't munmap %s (%d length)", artfile, len));
 								logd (("fetched %s (%d bytes) from cache", artfile, len));
 							}
-						} else /* fstat failed */
-							lockun (fd);
+						} else
 						close (fd);
 						fd = -1;
 					} /* revert, open _head failed */
@@ -638,7 +616,7 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 	if (fd<0)		/* cache off or request not in cache */
 	{
 		bool body;
-		int head_len=0; /* stop warning */
+		int head_len;
 		if ((!Cget (bfr, sizeof bfr) ||
 		     !(response=strToi (bfr))) || 
 		    (response != NNTP_ARTICLE_FOLLOWS_VAL &&
@@ -738,8 +716,8 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 			
 			/* if we pulled it from the cache, then this has already been added */
 			art_stack = strnStackAdd (art_stack, XCACHE, sizeof(XCACHE)-1);
-			head_len = art_stack->used-1;
 		}
+		head_len = art_stack? art_stack->used-1 : 0;
 		if (type == c_article || (type == c_body && CurrentScfg->article_timeout))
 			art_stack = strnStackAdd (art_stack, "\r\n", 2);
 		if (!f_filt)
@@ -773,9 +751,22 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 			 */
 			char *blocker;
 			if (type == c_body && !CurrentScfg->article_timeout) /* non cached filtered body has no head */
-				blocker = filterText(type, CurrentGroupAuth, NULL, 0, NULL, 0, art_stack->data, art_stack->used-1);
+				blocker = filterText(type,
+						     CurrentGroupAuth,
+						     NULL,
+						     0,
+						     NULL,
+						     0,
+						     art_stack->data, art_stack->used-1);
 			else
-				blocker = filterText(type, CurrentGroupAuth, art_stack->data, head_len, art_stack->data, art_stack->used-1, (type == c_body || type == c_article)? art_stack->data+head_len+2: NULL, (type == c_body || type == c_article)? art_stack->used-1-head_len-2: 0);
+				blocker = filterText(type,
+						     CurrentGroupAuth,
+						     art_stack->data,
+						     head_len,
+						     art_stack->data,
+						     art_stack->used-1,
+						     (type == c_body || type == c_article)? art_stack->data+head_len+2: NULL,
+						     (type == c_body || type == c_article)? art_stack->used-1-head_len-2: 0);
 			if (blocker)
 			{
 				logd (("content filter %s blocked %s of %s/%d", blocker, cmd->cmd, CurrentDir, artno));
@@ -816,14 +807,12 @@ EXPORT bool CMDarticle (struct command *cmd, char *args, bool dontcache)
 		int len = 0; /* stop -Wall complaining */
 		int wsize;
 		int t;
-		locksh (fd);
 		if (fstat (fd, &st) != 0 || st.st_size <2)
 			goto err;
 		len = st.st_size;
 		buf = (char *)mmap (0, len, PROT_READ, MAP_PRIVATE, fd, 0);
 		if (buf == (char *)-1)
 			goto err;
-		lockun (fd);
 		close (fd);
 		if (buf[len - 1] != '\n') /* truncated file */
 			goto err;
