@@ -11,7 +11,7 @@
  ************************************************************************/
 #ifdef RCS
 static const char rcsid[]=
- "$Id: bregex.c,v 1.1.1.1 1998/07/29 15:14:29 proff Exp $";
+ "$Id: bregex.c,v 1.2 2002/03/24 13:39:12 proff Exp $";
 #endif
 #include <sys/types.h>			/* size_t */
 #include <stddef.h>			/* offsetof() */
@@ -532,7 +532,7 @@ static void initsimp(rp,e)struct eps*rp,*e;
 #endif
 }
 		       /* we need one forward declaration to allow recursion */
-static por P((VSA const struct eps*const e));
+static int por P((VSA const struct eps*const e));
 
 #define IN_CLASS(CLASS,class)	\
  case CLASS:\
@@ -1031,7 +1031,7 @@ static void initjtable(VSO)VSD
   goodness bestval;jt_type*bt=Jtable(gpreg->re_finpoint);
   unsigned depth,maxdepth,th1,ot1;
   register struct eps*stack,*thiss,*other,*code,*s;jt_type jt[OPB];
-  ;{ register i=OPB-1;
+  ;{ register int i=OPB-1;
      while(jt[i]=0,i--);			  /* pre-init our jump table */
    }
   th1=ioffsetof(struct chclass,pos1);ot1=ioffsetof(struct chclass,pos2);
