@@ -1,12 +1,13 @@
-/* $Id: http.c,v 1.3 1998/08/04 20:18:54 proff Exp $
+/* $Id: http.c,v 1.6 2002/03/26 11:18:35 proff Exp $
  * $Copyright$
  */
 
 #include "nglobal.h"
+
 #include "acc.h"
 #include "group.h"
+#include "nlist.h"
 #include "ll.h"
-#include "list.h"
 
 #include "confused_runtime.h"
 
@@ -361,7 +362,7 @@ static void html_newsgroup(struct strStack *out, int argc, char **argv, struct n
 		*p = '-';
         }
     strStackAdd(out, "</tr>\n");
-};
+}
 
 #define MAC(x) \
 static void macro_ ## x (struct strStack *out, int argc, char **argv)
@@ -626,8 +627,8 @@ MAC(tasklist)
     for (tn=0; tn<=Stats->task_high; tn++)
 	{
 	    int n;
-	    task = TaskList[tn]; /* XXX atomic locking */
-	    if (!task)
+	    task = &TaskList[tn]; /* XXX atomic locking */
+	    if (task->ti_state == nc_none)
 		continue;
 	    strStackAdd(out, "<tr>");
 	    for (n=1; n<argc; n++)
@@ -661,20 +662,11 @@ MAC(newsgroups)
     pat = argv[1];
     macro_html_table (out, argc-2, argv+2);
     for (n=Ni->newsgroup_head; n; n=n->next)
-	{
-#ifdef READ_LOCKS
-	    if (!newsgroupLockRead(n))
-		continue;
-#endif
-	    if (!match (pat, n->group, 1, 0))
-		goto cont;
-	    html_newsgroup(out, argc-2, argv+2, n);
-	cont:
-#ifdef READ_LOCKS
-	    n->read_locks--;
-#else
-	    continue;
-#endif
+    {
+	    newsgroupLockRead(n);
+	    if (match (pat, n->group, 1, 0))
+		    html_newsgroup(out, argc-2, argv+2, n);
+	    newsgroupUnlockRead(n);
 	}
     strStackAdd(out, ENDTABLE);
 }
@@ -727,10 +719,10 @@ MAC(cputab)
     add(ENDTABLE);
 }
 
-#define S(x,y) {#x , macro_ ##x , y}
 
 struct macro_func macro[] = 
 {
+#define S(x,y) {#x , macro_ ##x , y}
     S(IPCfromChild, 0),
     S(IPCfromChildBytes, 0),
     S(IPCtoChild, 0),
@@ -774,8 +766,10 @@ struct macro_func macro[] =
     S(version, 0),
     S(cacheStats, 1),
     {NULL, NULL, 0}
+#undef S
 };
 
+
 static void macro_call (struct strStack *out, int argc, char **argv)
 {
     struct macro_func *s;
@@ -955,7 +949,7 @@ EXPORT bool httpHandler()
     if (!fillAuth(fileno(clientin), "<http>"))
 	{
 	    http_emit_header (HTTP_STATUS_FORBIDDEN,"Access Forbidden");
-	    emitf ("<H1>Access Forbidden</H1> <%s>, you do not have connect permissions in the %s file.<P>\r\n", ClientHost, con->accessFile);
+	    emitf ("<H1>Access Forbidden</H1> [%s], you do not have connect permissions in the %s file.<P>\r\n", ClientHost, con->accessFile);
 	    flush ();
 	    log (("refused connect from %s (%s)", ClientHost, ClientHostAddr));
 	    return FALSE;
