@@ -7,7 +7,8 @@
 
 EXPORT int lockex (int fd)
 {
-	int ret;
+	int ret = 0;
+#ifndef NO_LOCKING
 #ifdef LOCK_EX
 	ret = flock (fd, LOCK_EX);
 #else
@@ -16,13 +17,15 @@ EXPORT int lockex (int fd)
 #else
 #error no locking style for this system
 #endif
+#endif
 #endif
 	return ret;
 }
 
 EXPORT int locksh (int fd)
 {
-	int ret;
+	int ret = 0;
+#ifndef NO_LOCKING
 #ifdef LOCK_EX
 	ret = flock (fd, LOCK_SH);
 #else
@@ -31,13 +34,15 @@ EXPORT int locksh (int fd)
 #else
 #error no locking style for this system
 #endif
+#endif
 #endif
 	return ret;
 }
 
 EXPORT int lockun (int fd)
 {
-	int ret;
+	int ret = 0;
+#ifndef NO_LOCKING
 #ifdef LOCK_UN
 	ret = flock (fd, LOCK_UN);
 #else
@@ -46,6 +51,7 @@ EXPORT int lockun (int fd)
 #else
 #error no locking style for this system
 #endif
+#endif
 #endif
 	return ret;
 }
