@@ -1,4 +1,4 @@
-/* $Id: log.c,v 1.1.1.1 1998/07/29 15:14:29 proff Exp $ */
+/* $Id: log.c,v 1.7 2002/03/24 13:05:12 proff Exp $ */
 
 #include "nglobal.h"
 #include "log.h"
@@ -8,14 +8,19 @@
  * using vsprintf, I'd dearly like to know. CPP IS LAME.
  */
 
-EXPORT char *logPrintP;
+EXPORT char logPrintP[MAX_SYSLOG];
 
-EXPORT char *logPrint (char *fmt,...)
+EXPORT char *logPrint (char *fmt,...) EXP_(GNUC_EXT(__attribute__ ((format (printf, 1, 2)))))
 {
 	int i;
 	va_list argp;
 	va_start (argp, fmt);
 	i = vsnprintf (logPrintP, MAX_SYSLOG-1, fmt, argp);
 	va_end (argp);
+	if (Debug_fd >= 0)
+	{
+		write(Debug_fd, logPrintP, strlen(logPrintP));
+		write(Debug_fd, "\n", 1);
+	}
 	return logPrintP;
 }
