@@ -1,4 +1,4 @@
-/* $Id: nglobal.h,v 1.2 1998/08/02 20:35:02 proff Exp $
+/* $Id: nglobal.h,v 1.3 2000/05/23 14:48:40 proff Exp $
  * $Copyright$
  */
 
@@ -8,12 +8,13 @@
 #include "standard.h"
 #include "libproff.h"
 #include "nnconf.h"
+
 #include "assert.h"
 
 #include "nmalloc.h"
 
 #include "ll.h"
-#include "list.h"
+#include "nlist.h"
 #include "log.h"
 #include "nntp.h"
 #include "nntpcache.h"
