@@ -1,4 +1,4 @@
-/* $Id: nntpcache.h,v 1.2 1998/08/02 20:35:03 proff Exp $
+/* $Id: nntpcache.h,v 1.3 2000/05/21 15:05:19 proff Exp $
  * $Copyright$
  */
 
@@ -14,7 +14,8 @@ struct command
 };
 
 enum auth_state {none, user, valid};
-enum task_state {nc_none=0, nc_master, nc_client, nc_update, nc_expire, nc_nocem, nc_oneshot, nc_http, nc_last};
+/* keep the following in-sync with task_desc[] in nntpcache.c! */
+enum task_state {nc_none=0, nc_master, nc_client, nc_update, nc_expire, nc_nocem, nc_oneshot, nc_http, nc_watch, nc_last};
 
 struct task_info 
 {
