@@ -1,4 +1,4 @@
-/* $Id: xover.c,v 1.2 1998/08/02 20:35:04 proff Exp $
+/* $Id: xover.c,v 1.4 2002/03/29 09:50:58 proff Exp $
  * $Copyright$
  */
 
@@ -76,9 +76,9 @@ static struct xover_file
 	int outbuf_used;
 	int size;
 	int fd;
-	bool locked;
 	bool reader;
 	bool writer;
+	time_t mtime;
 } *xover_file_head = NULL, *xover_file_tail = NULL;
 
 static int xover_file_nodes = 0; /* number in cache */
@@ -98,19 +98,19 @@ EXPORT struct strList *overviewFmtGen(struct server_cfg *scfg, struct strList *l
     struct strList *xref = NULL;
     n_u32 hash;
     for (hash = 0xadeadfed; l; l = l->next)
-	{
+    {
 	    list=strXMListAdd(list, l->data);	/* place into shared memory */
 	    hash=strHash(hash, l->data);
 	    fmt_len += strlen(l->data) + 2;
 	    fmt_entries++;
 	    if (scfg && !xref && strnCaseEq(l->data, "Xref:", 5))
 		xref = l;
-	}
+    }
     if (list)
-	{
+    {
 	    list = list->head;
 	    if (scfg && (!scfg->share->overview_fmt_hash || scfg->share->overview_fmt_hash != hash))
-		{
+	    {
 		    struct strList *t;
 		    t = scfg->share->overview_fmt;
 		    scfg->share->overview_fmt = list->head;
@@ -119,8 +119,8 @@ EXPORT struct strList *overviewFmtGen(struct server_cfg *scfg, struct strList *l
 		    scfg->share->overview_fmt_xref_full = xref? strnCaseEq(xref->data, "Xref:full", 9): FALSE;
 		    if (t)
 			strXMListFree(t);
-		}
-	}
+	    }
+    }
     if (hashp)
 	*hashp = hash;
     return list;
@@ -452,41 +452,129 @@ static struct xover_file *xf_find (int i)
 	return NULL;
 }
 
-inline static void xf_release_reader (struct xover_file *xf)
+static bool xf_write_one (struct xover_file *xf)
 {
+	if (lseek (xf->fd, 0, SEEK_SET) != 0 ||
+	    write (xf->fd, xf->buf, 4 * MI) != 4 * MI ||
+	    (xf->outbuf &&
+	     (lseek (xf->fd, 0, SEEK_END) < 4 * MI ||
+	      write (xf->fd, xf->outbuf, xf->outbuf_used) != xf->outbuf_used)))
+	{
+		loge (("unable to write/lseek '%s/%s' correctly (...unlinking)", xf->dir, xf->name));
+		unlink (xf->name);
+		return FALSE;
+	}
+	xf->mtime = time (NULL);
+	return TRUE;
+}
+
+static bool xf_check_buf (struct xover_file *xf, bool readAll)
+{
+	struct stat st;
+	int size;
+
+	if (fstat (xf->fd, &st) < 0)
+	{
+		loge (("unable to fstat '%s/%s'", xf->dir, xf->name));
+		return FALSE;
+	}
+	if (st.st_size < MI * 4)
+	{
+		loge (("short file, %d bytes '%s/%s'", (int)st.st_size, xf->dir, xf->name));
+		return FALSE;
+	}
+	xf->size = st.st_size;
+	if (!xf->buf || st.st_mtime > xf->mtime)
+	{
+		if (lseek (xf->fd, 0, SEEK_SET) != 0)
+		{
+			loge (("couldn't lseek to start of '%s/%s'", xf->dir, xf->name));
+			return FALSE;
+		}
+		if (readAll)
+			size = xf->size;
+		else
+			size = MI * 4;
+		xf->buf = Srealloc (xf->buf, size);
+		if (read (xf->fd, xf->buf, size) != size)
+		{
+			loge (("couldn't read all of %d bytes from '%s/%s'", size, xf->dir, xf->name));
+			free (xf->buf);
+			xf->buf = 0;
+			return FALSE;
+		}
+	} else
+	{
+		if (!readAll)
+			return TRUE;
+		if (lseek (xf->fd, MI * 4, SEEK_SET) != MI * 4)
+		{
+			loge (("couldn't lseek to %d in '%s/%s'", MI * 4, xf->dir, xf->name));
+			return FALSE;
+		}
+		xf->buf = Srealloc (xf->buf, xf->size);
+		if (read (xf->fd, xf->buf + MI * 4, xf->size - MI * 4) != xf->size - MI * 4)
+		{
+			loge (("couldn't read all of %d bytes from '%s/%s'", xf->size - MI * 4, xf->dir, xf->name));
+			free (xf->buf);
+			xf->buf = 0;
+			return FALSE;
+		}
+	}
+	xf->mtime = st.st_mtime;
+	return TRUE;
+}
+
+#if 0		/* not needed at the moment */
+static bool xf_acquire_reader (struct xover_file *xf, bool readAll)
+{
+	if (xf->fd >= 0 && !xf->reader)
+		if (locksh (xf->fd) == 0)
+			if (!(xf->reader = xf_check_buf (xf, readAll)))
+				lockun (xf->fd);
+	return xf->reader;
+}
+#endif
+
+static void xf_release_reader (struct xover_file *xf)
+{
+	if (xf->fd >= 0 && xf->reader)
+		lockun (xf->fd);
 	xf->reader = FALSE;
 }
 
-static void xf_release_writer (struct xover_file *xf)
+static bool xf_acquire_writer (struct xover_file *xf, bool readAll)
 {
-        xf->writer = FALSE;
+	if (xf->reader)
+		xf_release_reader (xf);
+	if (xf->fd >= 0 && !xf->writer)
+		if (lockex (xf->fd) == 0)
+			if (!(xf->writer = xf_check_buf (xf, readAll)));
+				lockun (xf->fd);
+	return xf->writer;
+}
+
+static bool xf_release_writer (struct xover_file *xf)
+{
+	bool res = TRUE;
+	if (xf->fd > 0 && xf->writer && xf->buf)
+		res = xf_write_one (xf);
 	if (xf->outbuf)
 	{
-	    if (lseek (xf->fd, 0, SEEK_SET) != 0 || /* XXX consider optimizing down to one
-						     * write (and maybe one lseek) for the
-						     * initial case */
-		write (xf->fd, xf->buf, 4 * MI) != 4 * MI ||
-		lseek (xf->fd, 0, SEEK_END) < 4 * MI ||
-		write (xf->fd, xf->outbuf, xf->outbuf_used) != xf->outbuf_used)
-		{
-		    loge (("unable to write/lseek '%s/%s' correctly (...unlinking)", xf->dir, xf->name));
-		    unlink (xf->name);
-		}
 		free (xf->outbuf);
                 xf->outbuf = NULL;
 		xf->outbuf_used = 0;
         }
-	if (xf->locked)
-	{
+	if (xf->fd > 0 && xf->writer)
 		lockun (xf->fd);
-		xf->locked = FALSE;
-	}
+	xf->writer = FALSE;
+	return res;
 }
 
 static void xf_close (struct xover_file *xf)
 {
-        xf_release_reader (xf);
-        xf_release_writer (xf);
+	xf_release_reader (xf);
+	xf_release_writer (xf);
 	if (xf->name)
 	{
 		free (xf->name);
@@ -564,59 +652,12 @@ static void xf_destroy_all ()
 
 static void xf_release_all ()
 {
-    struct xover_file *xf = xover_file_tail;
-    for (; xf; xf = xf->prev)
-	    xf_release_writer (xf);
+	struct xover_file *xf = xover_file_tail;
+	for (; xf; xf = xf->prev)
+		if (!xf_release_writer (xf))
+			xf_close (xf);
 }
 
-
-#ifdef XPOSTED_XOVERS_BITE_BIG_RED_MARTIAN_SPORES_IN_TERMS_OF_DISK_PERFORMENCE
-#error so I didnt maintain them in the optimised version
-void xover_xpost ()
-{
-	/* an xover with Xrefs? */
-	if (!(xref = strstr (dat, "\tXref: ")))
-	{
-		return hisAdd (key, dat);
-	}
-	xref = Sstrdup (xref + 7);
-	p = strtok (xref, " ");	/* skip news server */
-	if (p)
-		p = first_xref = strtok (NULL, " ");
-	if (!p || !strchr (p, ':'))
-	{
-		free (xref);
-		return hisAdd (key, dat);
-	}
-	/* xrefs after the first just get pointers to the first one */
-	while ((p = strtok (NULL, " ")))
-	{
-		char *colon;
-		if (!(colon = strchr (p + 1, ':')))
-			continue;
-		hisAdd (p, first_xref);
-		if (strCaseEq (p, key))
-			f_done_key = TRUE;
-	}
-
-	/* the calling group:artnum didn't appear in the Xrefs */
-
-	if (!f_done_key && !strCaseEq (first_xref, key))
-		hisAdd (key, first_xref);
-
-	/* first xref gets all the xover data. We store it last though,
-	   because when the file is aged we can safely kull pointer
-	   xrefs upto this */
-	if (!hisAdd (first_xref, dat))
-	{
-		free (xref);
-		return FALSE;
-	}
-	free (xref);
-	return TRUE;
-}
-#endif
-
 static int xover_emit_xhdr (char *xover, int xn, struct strList *i_fmt, char *xhdr)
 {
 	int len;
@@ -635,17 +676,72 @@ static int xover_emit_xhdr (char *xover, int xn, struct strList *i_fmt, char *xh
 	return len;
 }
 
+static void xf_grab_outbuf (struct xover_file *xf, int len)
+{
+#define GRAB_SIZE 32768
+	if (!xf->outbuf)
+	{
+		xf->outbuf = Smalloc (GRAB_SIZE);
+		xf->outbuf_len = GRAB_SIZE;
+	} else if (xf->outbuf_used + len > xf->outbuf_len)
+		xf->outbuf = Srealloc (xf->outbuf, (xf->outbuf_len += GRAB_SIZE));
+#undef GRAB_SIZE
+}
+
+static struct xover_file *xf_open (int index)
+{
+	int fd;
+	char fn[64];
+	struct stat st;
+	struct xover_file *xf;
+
+	sprintf (fn, "%d_xover", index * MI);
+	fd = open (fn, O_RDWR | O_CREAT, 0664);
+	if (fd < 0)
+	{
+		loge (("unable to open/create '%s/%s'", CurrentDir, fn));
+		return NULL;
+	}
+	xf = xf_add (index);
+	xf->fd = fd;
+	xf->name = Sstrdup (fn);
+	xf->dir = Sstrdup (CurrentDir);
+	if (lockex (xf->fd) != 0)
+	{
+		loge (("unable to lockex '%s/%s' (...unlinking)", xf->dir, xf->name));
+		unlink (xf->name);
+		xf_close (xf);
+		return NULL;
+	}
+	if (fstat (xf->fd, &st) != 0)
+	{
+		loge (("unable to fstat '%s/%s' (...unlinking)", xf->dir, xf->name));
+		unlink (xf->name);
+		lockun (xf->fd);
+		xf_close (xf);
+		return NULL;
+	}
+	if (st.st_size < MI * 4 && ftruncate (xf->fd, MI * 4) != 0)
+	{
+		loge (("unable to truncate '%s/%s' (...unlinking)", xf->dir, xf->name));
+		unlink (xf->name);
+		lockun (xf->fd);
+		xf_close (xf);
+		return NULL;
+	}
+	lockun (xf->fd);
+	return xf;
+}
+
 static bool xover_input (char *xhdr, int min, int max)
 {
 	int len;
 	int response;
 	char *p;
 	struct xover_file *xf;
-	struct stat st;
 	int t;
 	n_u32 *idx;
 	n_u32 i32;
-	char fn[64]; /* n_xover */
 	char xover[MAX_XOVER + 20];
 	bool f_translated;
 	if (!(len=Cget (xover, sizeof xover)))
@@ -718,35 +814,11 @@ static bool xover_input (char *xhdr, int min, int max)
 	t = response / MI;
 	if (!(xf = xf_find (t)))
 	{
-		int fd;
-		int base;
-		sprintf (fn, "%d_xover", t * MI);
-		fd = open (fn, O_RDWR | O_CREAT, 0664);
-		if (fd < 0 || fstat (fd, &st) < 0)
-		{
-			loge (("unable to open/create/fstat '%s/%s'", CurrentDir, fn));
-	
-		}
-		if (lockex (fd) < 0)
-		{
-			loge (("unable to lockex '%s/%s'", CurrentDir, fn));
+		if (!(xf = xf_open (t)))
 			goto auth_xover;
-		}
-		xf = xf_add (t);
-		xf->fd = fd;
-		xf->name = Sstrdup (fn);
-		xf->dir = Sstrdup (CurrentDir);
-		xf->buf = Scalloc (1, MI * 4);
-		xf->locked = TRUE;
-		xf->size = st.st_size;
-		if (xf->size > 0 && read (xf->fd, xf->buf, MI * 4) != MI * 4)
-		{
-		        /* this xover file has seen better days */
-			logw (("xover file was way too short '%s/%s' (...truncated)", xf->dir, xf->name));
-			ftruncate (fd, 0);
-			xf->size = 0;
-			memset (xf->buf, 0, MI *4);
-		}
+
+#if 0
+		/* This is now always done in xover_process. */
 		/*
 		 * set failed flag in index for xover region asked.
 		 * we then knock em out one by one as xovers come in
@@ -782,24 +854,10 @@ static bool xover_input (char *xhdr, int min, int max)
 		 */
 		base = (min<t*MI)? 0: min%MI;
 		memset (xf->buf+base*4, 0xff, (MIN(MI, max+1-t*MI)-base)*4);
-	} else
-	{
-		if (!xf->locked) /* size can't change if we have locked it */
-		{
-			if (lockex (xf->fd) < 0)
-			{
-				loge (("unable to lockex '%s/%s'", xf->dir, xf->name));
-			}
-			xf->locked = TRUE;
-			if (fstat (xf->fd, &st) < 0)
-			{
-				loge (("unable to fstat '%s/%s'", xf->dir, xf->name));
-				xf_release_writer (xf);
-				goto auth_xover;
-			}
-			xf->size = st.st_size;
-		}
+#endif
 	}
+	if (!xf_acquire_writer (xf, FALSE))
+		goto auth_xover;
 	len = strlen (p) + 1;
 	idx = (n_u32 *) xf->buf;
 	i32 = idx[response % MI];
@@ -822,13 +880,7 @@ static bool xover_input (char *xhdr, int min, int max)
 	if (i32 == 0 || i32 == 0xffffffff)	/* only if we don't have it already */
 	{
 		idx[response % MI] = htonl ((len << 20) | (((xf->size > 0) ? xf->size : MI * 4) + xf->outbuf_used));
-#define GRAB_SIZE 32768
-		if (!xf->outbuf)
-		{
-			xf->outbuf = Smalloc (GRAB_SIZE);
-			xf->outbuf_len = GRAB_SIZE;
-		} else if (xf->outbuf_used + len > xf->outbuf_len)
-			xf->outbuf = Srealloc (xf->outbuf, (xf->outbuf_len += GRAB_SIZE));
+		xf_grab_outbuf(xf, len);
 		memcpy (xf->outbuf + xf->outbuf_used, p, len);
 		xf->outbuf_used += len;
 	}
@@ -839,7 +891,6 @@ auth_xover:
 	{
 		logd (("content filter %s blocked xover of %s/%d", blocker, CurrentDir, response));
 		CS->filter_blocked++;
-
 		return TRUE;
 	}
 	if (f_translated)
@@ -856,7 +907,6 @@ auth_xover:
 			emitrn (xover);
 	}}
 	return TRUE;
-#undef GRAB_SIZE
 }
 
 static void xover_output (int min, int max)
@@ -912,46 +962,15 @@ static bool xover_process (int min, int max, char *xhdr)
 	for (xi = xmin; xi <= xmax; xi++)
 	{
 		char xover[20 + MAX_XOVER];
-		struct stat st;
 		int start, end;
 		n_u32 *idx;
 		char *xover_buf = NULL;		/* not null when entire xover db file in incore */
 		int n;
 		int xn;
 		struct xover_file *xf;
-		bool xf_cached;
-		if (!(xf = xf_find (xi)))
-		{
-			char fn[64];
-			int fd;
-			sprintf (fn, "%d_xover", xi * MI);
-			fd = open (fn, O_RDWR, 0664);	/* RDWR so we can perhaps use the fd later for a write */
-			if (fd < 0)
-				continue;
-		        xf = xf_add (xi);
-			xf->fd = fd;
-			xf->name = Sstrdup (fn);
-			xf->dir = Sstrdup (CurrentDir);
-			xf_cached = FALSE;
-		} else
-		        xf_cached = TRUE;
-		if (fstat (xf->fd, &st) < 0)
-		{
-			loge (("unable to fstat '%s/%s' (...unlinked)", xf->dir, xf->name));
-			goto bad;
-		}
-		if (st.st_size <= MI * 4)
-		{
-			if (!xf_cached)
-			{
-				loge (("short file, %d bytes '%s/%s' cwd %s (...unlinked)", st.st_size, xf->dir, xf->name, CurrentDir));
-				goto bad;
-			}
-                        xf_release_reader (xf);
-                        continue;
-                }
-		xf->size = st.st_size;
 
+		 if (!(xf = xf_find (xi)) && !(xf = xf_open (xi)))
+		 	continue;
 		/*
 		 * we have two techniques depending on how much of
 		 * the xover file is required. if we require more than 16
@@ -961,39 +980,10 @@ static bool xover_process (int min, int max, char *xhdr)
 		 */
 		start = (xi == xmin) ? min % MI : 0;
 		end = (xi == xmax) ? max % MI : MI - 1;		/* upto and including */
-		if (end - start < 16)
-		{
-		        if (!xf_cached)
-			{
-				xf->buf = Smalloc (MI * 4);
-				if (read (xf->fd, xf->buf, MI * 4) != MI * 4)
-				{
-					loge (("couldn't read all %d bytes from '%s/%s' (...unlinked)", MI * 4, xf->dir, xf->name));
-					goto bad;
-				}
-			}
-		} else
-		{
-		        int rb;
-			if (xf_cached)
-                        {
-				if (lseek (xf->fd, 4 * MI, SEEK_SET) != 4 * MI)
-                                {
-					loge (("couldn't lseek to %d in '%s/%s' (...unlinked)", MI * 4, xf->dir, xf->name));
-                                        goto bad;
-                                }
-				xf->buf = Srealloc (xf->buf, xf->size);
-                        }
-			else
-				xf->buf = Smalloc (xf->size);
-			rb = xf_cached? xf->size - 4*MI: xf->size;
-			if (read (xf->fd, xf_cached? xf->buf + 4*MI: xf->buf, rb) != rb)
-			{
-				loge (("couldn't read all of %d bytes from '%s/%s' (...unlinked)", rb, xf->dir, xf->name));
-				goto bad;
-			}
-			xover_buf = xf->buf + 4 * MI;
-		}
+		if (!xf_acquire_writer (xf, end - start >= 16))
+			goto bad;
+		if (end - start >= 16)
+			xover_buf = xf->buf + MI * 4;
 		idx = (n_u32 *) (xf->buf);
 		for (n = start, xn = start + MI * xi, colmax = 0; n <= end; n++, xn++)
 		{
@@ -1059,16 +1049,16 @@ static bool xover_process (int min, int max, char *xhdr)
 			if (CurrentGroupNode->hi_xover < xn)
 			    CurrentGroupNode->hi_xover = xn;
 			if (FILT && (blocker = xover_filter(xov, overviewFmt)))
-			    {
+			{
 				logd (("content filter %s blocked xover of '%s/%d'", blocker, CurrentDir, xn));
 				CS->filter_blocked++;
-			    }
+			}
 			else
 			{
-			    if (xhdr)
-				xover_emit_xhdr (xov, xn, overviewFmt, xhdr);
-			    else
-				emitf ("%d\t%s\r\n", xn, xov);
+				if (xhdr)
+					xover_emit_xhdr (xov, xn, overviewFmt, xhdr);
+				else
+					emitf ("%d\t%s\r\n", xn, xov);
 			}
 		}
 		if (xover_buf)
@@ -1199,7 +1189,7 @@ EXPORT bool CMDxhdr (char *args)
 
 	if (sscanf (args, "%*s %127s", header) != 1)
 	{
-	    emitrn (NNTP_SYNTAX_USE);
+	 	emitrn (NNTP_SYNTAX_USE);
 	    bad:
 		return FALSE;
 	}
@@ -1211,7 +1201,7 @@ EXPORT bool CMDxhdr (char *args)
 	}
 	if (!*msgid && (!*CurrentGroup || !setGD()))
 	{
-	    emitrn (NNTP_NOTINGROUP);
+		emitrn (NNTP_NOTINGROUP);
 		goto bad;
 	}
 	sprintf(field, "%.126s:", header);
@@ -1226,8 +1216,8 @@ EXPORT bool CMDxhdr (char *args)
 		Cflush ();
 		if (con->maxMsgIDsearch<1 || !Cget (buf, sizeof buf))
 		{
+			CurrentScfg->share->xhdr_fail++;
 			emitrn (NNTP_SERVERTEMPDOWN);
-		        CurrentScfg->share->xhdr_fail++;
 			return FALSE;
 		}
 		if (strToi (buf) == NNTP_HEAD_FOLLOWS_VAL)
